/* 
 *  Copyright 1992 Rainer Klute <klute@irb.informatik.uni-dortmund.de>
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appear in all copies and that both that copyright
 *  notice and this permission notice appear in supporting documentation.  The
 *  author makes no representations about the suitability of this software for
 *  any purpose.  It is provided "as is" without express or implied warranty.
 */

#include <setjmp.h>
#include <stdio.h>

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>

#if defined (MotifWidgets)
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Text.h>
#elif defined (AthenaWidgets)
#include <X11/Shell.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/AsciiText.h>
#endif

#include "patchlevel.h"




#ifdef DEBUG
#define dprintf printf
#else
#define dprintf(a) /* as nothing (hack, hack) */
#endif



#ifdef __STDC__
static int ErrorHandler (Display *, XErrorEvent *);
static int IOErrorHandler (Display *);
static XtEventHandler ClientMessageHandler (Widget, XtPointer, XEvent *,
                                            Boolean *);
static XtCallbackProc Close (Widget, XtPointer, XtPointer);
static DestroyWidget (Widget);
static XtCallbackProc Kill (Widget, XtPointer, XtPointer);
static NewTopLevelWidget (Display *);
static XtCallbackProc NewWidget (Widget, XtPointer, XtPointer);
static XtCallbackProc OpenDisplay (Widget, XtPointer, XtPointer);
static XtCallbackProc Quit ();
#else
static int ErrorHandler ();
static int IOErrorHandler ();
static XtEventHandler ClientMessageHandler ();
static XtCallbackProc Close ();
static DestroyWidget ();
static XtCallbackProc Kill ();
static NewTopLevelWidget ();
static XtCallbackProc NewWidget ();
static XtCallbackProc OpenDisplay ();
static XtCallbackProc Quit ();
#endif



jmp_buf xioRecover;



static DestroyWidget (widget)
Widget widget;
{
    dprintf ("destroying widget on display \"%s\"\n", DisplayString
	     (XtDisplay (widget)));
    XtDestroyWidget (widget);
}




/* ARGSUSED */

static XtCallbackProc Quit (widget, clientData, callData)
Widget widget;
XtPointer clientData, callData;
{
    XtDestroyApplicationContext (XtWidgetToApplicationContext (widget));
    exit (0);
}




/* ARGSUSED */

static XtCallbackProc Close (widget, clientData, callData)
Widget widget;
XtPointer clientData, callData;
{
    while (!XtIsShell (widget))
	widget = XtParent (widget);
    DestroyWidget (widget);
}




/* ARGSUSED */

static XtCallbackProc Kill (widget, clientData, callData)
Widget widget;
XtPointer clientData, callData;
{
    dprintf ("killing display connection %d to \"%s\"\n", ConnectionNumber
	     (XtDisplay (widget)), DisplayString (XtDisplay (widget)));
    XKillClient (XtDisplay (widget), (XID) XtWindow (widget));
}




/* ARGSUSED */

static XtCallbackProc OpenDisplay (widget, clientData, callData)
Widget widget;
XtPointer clientData, callData;
{
    Widget t = (Widget) clientData;
    XtAppContext appContext = XtWidgetToApplicationContext (widget);
    Display *display;
    char *displayString;
    int argc = 0;

#if defined (MotifWidgets)
    displayString = XmTextGetString (t);
#elif defined (AthenaWidgets)
    XtVaGetValues (t, XtNstring, &displayString, NULL);
#endif
    display = XtOpenDisplay (appContext, displayString, NULL, "Mdisp", NULL,
			     0, &argc, NULL);
    if (display == NULL)
	return;
    dprintf ("connection to \"%s\" established\n", DisplayString (display));
    NewTopLevelWidget (display);
}




/* ARGSUSED */

static XtCallbackProc NewWidget (widget, clientData, callData)
Widget widget;
XtPointer clientData, callData;
{
    NewTopLevelWidget (XtDisplay (widget));
}




/* ARGSUSED */

static XtEventHandler ClientMessageHandler (widget, client_data, event,
					    continue_to_dispatch)
Widget widget;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
    switch (event->type)
    {
	case ClientMessage:
	    if (event->xclient.message_type ==
		XInternAtom (XtDisplay (widget), "WM_PROTOCOLS", True) &&
		event->xclient.data.l[0] ==
		XInternAtom (XtDisplay (widget), "WM_DELETE_WINDOW", True))
	    {
		dprintf ("delete window on display \"%s\"\n", DisplayString
			 (XtDisplay (widget)));
		DestroyWidget (widget);
	    }
    }
}




static NewTopLevelWidget (display)
Display *display;
{
    Widget toplevel, rowcolumn, text, button;
    char *displayString = DisplayString (display);

    toplevel =
    XtVaAppCreateShell (displayString, "Mdisp", applicationShellWidgetClass,
			display, NULL);
#if defined (MotifWidgets)
    XtVaSetValues (toplevel, XmNdeleteResponse, XmDO_NOTHING, NULL);
#endif
    XtAddEventHandler (toplevel, NoEventMask, True, (XtEventHandler)
		       ClientMessageHandler, NULL);

/*
 * Container for all the other widgets:
 */
#if defined (MotifWidgets)
    rowcolumn = 
    XtVaCreateManagedWidget ("rowcolumn", xmRowColumnWidgetClass, toplevel,
			     NULL);
#elif defined (AthenaWidgets)
    rowcolumn = 
    XtVaCreateManagedWidget ("rowcolumn", boxWidgetClass, toplevel,
			     NULL);
#endif

/*
 * Text widget for display string:
 */
#if defined (MotifWidgets)
    text = 
    XtVaCreateManagedWidget ("text", xmTextWidgetClass, rowcolumn,
			     XmNvalue, displayString,
			     NULL);
#elif defined (AthenaWidgets)
    text = 
    XtVaCreateManagedWidget ("text", asciiTextWidgetClass, rowcolumn,
			     XtNstring, displayString,
			     XtNeditType, XawtextEdit,
			     XtNresize, XawtextResizeWidth,
			     NULL);
#endif

/*
 * "Open display" button:
 */
#if defined (MotifWidgets)
    button =
    XtVaCreateManagedWidget ("Open display", xmPushButtonWidgetClass,
			     rowcolumn, NULL);
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc)
		   OpenDisplay, text);
#elif defined (AthenaWidgets)
    button =
    XtVaCreateManagedWidget ("Open display", commandWidgetClass, rowcolumn,
			     NULL);
    XtAddCallback (button, XtNcallback, (XtCallbackProc) OpenDisplay, text);
#endif

/*
 * "New widget" button:
 */
#if defined (MotifWidgets)
    button =
    XtVaCreateManagedWidget ("New widget", xmPushButtonWidgetClass,
			     rowcolumn, NULL);
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc) NewWidget,
		   NULL);
#elif defined (AthenaWidgets)
    button =
    XtVaCreateManagedWidget ("New widget", commandWidgetClass, rowcolumn,
			     NULL);
    XtAddCallback (button, XtNcallback, (XtCallbackProc) NewWidget, NULL);
#endif

/*
 * "Close" button:
 */
#if defined (MotifWidgets)
    button =
    XtVaCreateManagedWidget ("Close", xmPushButtonWidgetClass, rowcolumn,
			     NULL);
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc) Close,
		   NULL);
#elif defined (AthenaWidgets)
    button =
    XtVaCreateManagedWidget ("Close", commandWidgetClass, rowcolumn, NULL);
    XtAddCallback (button, XtNcallback, (XtCallbackProc) Close, NULL);
#endif

/*
 * "Kill" button:
 */
#if defined (MotifWidgets)
    button =
    XtVaCreateManagedWidget ("Kill", xmPushButtonWidgetClass, rowcolumn,
			     NULL);
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc) Kill,
		   NULL);
#elif defined (AthenaWidgets)
    button =
    XtVaCreateManagedWidget ("Kill", commandWidgetClass, rowcolumn, NULL);
    XtAddCallback (button, XtNcallback, (XtCallbackProc) Kill, NULL);
#endif

/*
 * "Quit" button:
 */
#if defined (MotifWidgets)
    button =
    XtVaCreateManagedWidget ("Quit", xmPushButtonWidgetClass, rowcolumn,
			     NULL);
    XtAddCallback (button, XmNactivateCallback, (XtCallbackProc) Quit,
		   NULL);
#elif defined (AthenaWidgets)
    button =
    XtVaCreateManagedWidget ("Quit", commandWidgetClass, rowcolumn, NULL);
    XtAddCallback (button, XtNcallback, (XtCallbackProc) Quit, NULL);
#endif

    XtRealizeWidget (toplevel);
}




/* ARGSUSED */

static int ErrorHandler (display, error)
Display *display;
XErrorEvent *error;
{
    dprintf ("X protocol error on display \"%s\"\n", DisplayString (display));
    XtCloseDisplay (display);
}




static int IOErrorHandler (display)
Display *display;
{
    dprintf ("IO error on display \"%s\"\n", DisplayString (display));
    XtCloseDisplay (display);
    longjmp (xioRecover, 1);
}




main (argc, argv)
int argc;
char *argv[];
{
    XtAppContext appContext;
    Display *display;

    XtToolkitInitialize ();
    XSetErrorHandler (ErrorHandler);
    XSetIOErrorHandler (IOErrorHandler);
    appContext = XtCreateApplicationContext ();
    display = XtOpenDisplay (appContext, NULL, argv[0], "Mdisp", NULL, 0,
			     &argc, argv);
    if (display == NULL)
	exit (1);
    NewTopLevelWidget (display);
    setjmp (xioRecover);
    XtAppMainLoop (appContext);
    return (0);   /* just to make lint happy */
}
